<?php

namespace PandoraFMS\Modules\Profiles\Controllers;

use PandoraFMS\Modules\Profiles\Actions\ListProfileAction;
use PandoraFMS\Modules\Profiles\Entities\ProfileFilter;
use PandoraFMS\Modules\Shared\Controllers\Controller;
use PandoraFMS\Modules\Shared\Services\ValidateAclSystem;

use Psr\Http\Message\ResponseInterface as Response;
use Psr\Http\Message\ServerRequestInterface as Request;

final class ListProfileController extends Controller
{


    public function __construct(
        private ListProfileAction $listProfileAction,
        private ValidateAclSystem $acl,
    ) {
    }


    /**
     * @OA\Post(
     *   security={{ "bearerAuth": {}}},
     *   tags={"Profiles"},
     *   path="/profile/list",
     *   summary="List profiles",
     * @OA\Parameter(ref="#/components/parameters/parameterPage"),
     * @OA\Parameter(ref="#/components/parameters/parameterSizePage"),
     * @OA\Parameter(ref="#/components/parameters/parameterSortField"),
     * @OA\Parameter(ref="#/components/parameters/parameterSortDirection"),
     * @OA\RequestBody(ref="#/components/requestBodies/requestBodyProfileFilter"),
     * @OA\Response(
     *     response="200",
     *     description="List Incidence object",
     *     content={
     * @OA\MediaType(
     *         mediaType="application/json",
     * @OA\Schema(
     * @OA\Property(
     *             property="paginationData",
     *             type="object",
     *             ref="#/components/schemas/paginationData",
     *             description="Page object",
     *           ),
     * @OA\Property(
     *             property="data",
     *             type="array",
     * @OA\Items(
     *               ref="#/components/schemas/Profile",
     *               description="Array of incidences Type objects"
     *             )
     *           ),
     *         ),
     *       )
     *     }
     *   ),
     * @OA\Response(response=400,                                                  ref="#/components/responses/BadRequest"),
     * @OA\Response(response=401,                                                  ref="#/components/responses/Unauthorized"),
     * @OA\Response(response=403,                                                  ref="#/components/responses/Forbidden"),
     * @OA\Response(response=404,                                                  ref="#/components/responses/NotFound"),
     * @OA\Response(response=500,                                                  ref="#/components/responses/InternalServerError")
     *  )
     */
    public function __invoke(Request $request, Response $response): Response
    {
        // @var ProfileFilter $profileFilter.
        $profileFilter = $this->fromRequest($request, ProfileFilter::class);

        $this->acl->validateUserAdmin();
        $this->acl->validate(0, 'UM', ' tried to manage profile');

        $result = $this->listProfileAction->__invoke($profileFilter);
        return $this->getResponse($response, $result);
    }


}
