<?php

namespace PandoraFMS\Modules\Tags\Controllers;

use PandoraFMS\Modules\Tags\Actions\ListTagAction;
use PandoraFMS\Modules\Tags\Entities\TagFilter;
use PandoraFMS\Modules\Shared\Controllers\Controller;
use PandoraFMS\Modules\Shared\Services\ValidateAclSystem;

use Psr\Http\Message\ResponseInterface as Response;
use Psr\Http\Message\ServerRequestInterface as Request;

final class ListTagController extends Controller
{


    public function __construct(
        private ListTagAction $listTagAction,
        private ValidateAclSystem $acl,
    ) {
    }


    /**
     * @OA\Post(
     *   security={{ "bearerAuth": {}}},
     *   tags={"Tags"},
     *   path="/tag/list",
     *   summary="List tags",
     * @OA\Parameter(ref="#/components/parameters/parameterPage"),
     * @OA\Parameter(ref="#/components/parameters/parameterSizePage"),
     * @OA\Parameter(ref="#/components/parameters/parameterSortField"),
     * @OA\Parameter(ref="#/components/parameters/parameterSortDirection"),
     * @OA\RequestBody(ref="#/components/requestBodies/requestBodyTagFilter"),
     * @OA\Response(
     *     response="200",
     *     description="List Incidence object",
     *     content={
     * @OA\MediaType(
     *         mediaType="application/json",
     * @OA\Schema(
     * @OA\Property(
     *             property="paginationData",
     *             type="object",
     *             ref="#/components/schemas/paginationData",
     *             description="Page object",
     *           ),
     * @OA\Property(
     *             property="data",
     *             type="array",
     * @OA\Items(
     *               ref="#/components/schemas/Tag",
     *               description="Array of incidences Type objects"
     *             )
     *           ),
     *         ),
     *       )
     *     }
     *   ),
     * @OA\Response(response=400,                                              ref="#/components/responses/BadRequest"),
     * @OA\Response(response=401,                                              ref="#/components/responses/Unauthorized"),
     * @OA\Response(response=403,                                              ref="#/components/responses/Forbidden"),
     * @OA\Response(response=404,                                              ref="#/components/responses/NotFound"),
     * @OA\Response(response=500,                                              ref="#/components/responses/InternalServerError")
     *  )
     */
    public function __invoke(Request $request, Response $response): Response
    {
        // @var TagFilter $tagFilter.
        $tagFilter = $this->fromRequest($request, TagFilter::class);

        $this->acl->validate(0, 'PM', ' tried to manage tag');

        $result = $this->listTagAction->__invoke($tagFilter);
        return $this->getResponse($response, $result);
    }


}
